@@ -26,15 +26,16 @@ a DB Engine and storage), and the main peer relation name will be
 import logging
 from typing import ClassVar, Generic, TypeVar
 
+import ops.log
 from data_platform_helpers.advanced_statuses.handler import StatusHandler
 from data_platform_helpers.advanced_statuses.models import StatusObject
 from data_platform_helpers.advanced_statuses.protocol import ManagerStatusProtocol
 from data_platform_helpers.advanced_statuses.types import Scope
 from ops.charm import CharmBase
 
-from single_kernel_mongo.config.literals import Substrates
+from single_kernel_mongo.config.literals import CharmKind, Substrates
 from single_kernel_mongo.config.relations import PeerRelationNames
-from single_kernel_mongo.config.statuses import CharmStatuses
+from single_kernel_mongo.config.statuses import CharmStatuses, MongoDBStatuses
 from single_kernel_mongo.core.operator import OperatorProtocol
 from single_kernel_mongo.core.structured_config import MongoConfigModel, MongoDBRoles
 from single_kernel_mongo.events.lifecycle import LifecycleEventsHandler
@@ -43,6 +44,8 @@ T = TypeVar("T", bound=MongoConfigModel)
 U = TypeVar("U", bound=OperatorProtocol)
 
 logger = logging.getLogger(__name__)
+logging.getLogger("httpx").setLevel(logging.WARNING)
+logging.getLogger("httpcore").setLevel(logging.WARNING)
 
 
 class AbstractMongoCharm(ManagerStatusProtocol, Generic[T, U], CharmBase):
@@ -69,6 +72,11 @@ class AbstractMongoCharm(ManagerStatusProtocol, Generic[T, U], CharmBase):
         # Init the Juju object Object
         super(Generic, self).__init__(*args)
 
+        root_logger = logging.getLogger()
+        for handler in root_logger.handlers:
+            if isinstance(handler, ops.log.JujuLogHandler):
+                handler.setFormatter(logging.Formatter("{name}:{message}", style="{"))
+
         # Create the operator instance (one of MongoDBOperator or MongosOperator)
         self.operator = self.operator_type(self)
         self.state = self.operator.state
@@ -104,13 +112,21 @@ class AbstractMongoCharm(ManagerStatusProtocol, Generic[T, U], CharmBase):
             )
             self.workload.install()
 
-    def on_leader_elected(self, _):
+    def on_leader_elected(self, event):
         """First leader elected handler."""
         # Sets the role in the databag: when the charm is first created, its
         # role won't exist in the databag. We save it in the databag because we
         # don't allow role changing yet.
+        if (
+            self.operator.name == CharmKind.MONGOD
+            and self.parsed_config.role == MongoDBRoles.INVALID
+        ):
+            self.status_handler.set_running_status(MongoDBStatuses.INVALID_ROLE.value, scope="app")
+            event.defer()
+            return
+
         if self.operator.state.app_peer_data.role == MongoDBRoles.UNKNOWN:
-            self.operator.state.app_peer_data.role = self.parsed_config.role
+            self.operator.state.app_peer_data.role = MongoDBRoles(self.parsed_config.role)
 
     def get_statuses(self, scope: Scope, recompute: bool = False) -> list[StatusObject]:
         """Returns a list of statuses."""
