@@ -6,7 +6,7 @@ This module should contain the literals used in the charms (paths, enums, etc).
 """
 
 from dataclasses import dataclass
-from enum import Enum
+from enum import Enum, IntEnum
 from pathlib import Path
 from typing import Generic, TypeVar
 
@@ -34,15 +34,15 @@ class Scope(str, Enum):
     UNIT = "unit"
 
 
-class MongoPorts(int, Enum):
+class MongoPorts(IntEnum):
     """The default Mongo ports."""
 
     MONGODB_PORT = 27017
     MONGOS_PORT = 27018
 
 
-class InternalUsers(str, Enum):
-    """The three allowed internal users."""
+class InternalUsernames(str, Enum):
+    """The allowed internal usernames."""
 
     OPERATOR = "operator"
     BACKUP = "backup"
@@ -50,28 +50,9 @@ class InternalUsers(str, Enum):
     LOGROTATE = "logrotate"
 
 
-class UnitState(str, Enum):
-    """Unit upgrade state."""
+SECRETS_APP = [f"{user}-password" for user in InternalUsernames] + ["keyfile"]
 
-    HEALTHY = "healthy"
-    RESTARTING = "restarting"  # Kubernetes only
-    UPGRADING = "upgrading"  # Machines only
-    OUTDATED = "outdated"  # Machines only
-
-
-SECRETS_APP = [f"{user}-password" for user in InternalUsers] + ["keyfile"]
-
-
-@dataclass(frozen=True)
-class Snap:
-    """The Snap related information."""
-
-    name: str = "charmed-mongodb"
-    channel: str = "6/edge"
-    revision: str = "129"
-
-
-SNAP = Snap(channel="6/edge", revision="129")
+VERSIONS_FILE = Path("refresh_versions.toml")
 
 T = TypeVar("T", bound=str | int)
 
@@ -109,7 +90,7 @@ MAX_PASSWORD_LENGTH = 4096
 
 PBM_RESTART_DELAY = 5
 
-FEATURE_VERSION_6 = "6.0"
+FEATURE_VERSION = "8.0"
 
 
 OS_REQUIREMENTS = {
@@ -123,3 +104,4 @@ class TrustStoreFiles(str, Enum):
     """The different files we store in the trust store."""
 
     PBM = "pbm.crt"
+    LDAP = "ldap.crt"
