@@ -21,6 +21,7 @@ from logging import getLogger
 from pathlib import Path
 from typing import TYPE_CHECKING, ClassVar, TypeAlias
 
+import charm_refresh
 from data_platform_helpers.advanced_statuses.models import StatusObject
 from data_platform_helpers.advanced_statuses.protocol import ManagerStatusProtocol
 from ops.charm import RelationDepartedEvent
@@ -49,10 +50,10 @@ if TYPE_CHECKING:
     from single_kernel_mongo.abstract_charm import AbstractMongoCharm
     from single_kernel_mongo.events.database import DatabaseEventsHandler
     from single_kernel_mongo.events.tls import TLSEventsHandler
-    from single_kernel_mongo.events.upgrades import UpgradeEventHandler
     from single_kernel_mongo.managers.ldap import LDAPManager
     from single_kernel_mongo.managers.tls import TLSManager
-    from single_kernel_mongo.managers.upgrade import MongoUpgradeManager
+    from single_kernel_mongo.managers.upgrade_v3 import MongoDBUpgradesManager
+    from single_kernel_mongo.managers.upgrade_v3_status import MongoDBUpgradesStatusManager
 
 logger = getLogger(__name__)
 
@@ -79,13 +80,14 @@ class OperatorProtocol(ABC, Object, ManagerStatusProtocol):
     config_manager: FileBasedConfigManager
     tls_manager: TLSManager
     state: CharmState
+    refresh: charm_refresh.Common | None
     mongo_manager: MongoManager
-    upgrade_manager: MongoUpgradeManager
+    upgrades_manager: MongoDBUpgradesManager
+    upgrades_status_manager: MongoDBUpgradesStatusManager
     ldap_manager: LDAPManager
     workload: MainWorkloadType
     client_events: DatabaseEventsHandler
     tls_events: TLSEventsHandler
-    upgrade_events: UpgradeEventHandler
     ldap_events: LDAPEventHandler
 
     if TYPE_CHECKING:
@@ -173,7 +175,7 @@ class OperatorProtocol(ABC, Object, ManagerStatusProtocol):
         ...
 
     @abstractmethod
-    def is_relation_feasible(self, name: str) -> bool:
+    def get_relation_feasible_status(self, name: str) -> StatusObject | None:
         """Checks if the relation is feasible in this context."""
         ...
 
@@ -215,6 +217,13 @@ class OperatorProtocol(ABC, Object, ManagerStatusProtocol):
                 "Scaling down the application, no need to process removed relation in broken hook."
             )
 
+    @property
+    def refresh_in_progress(self) -> bool:
+        """Check if charm-refresh is currently in progress."""
+        # If charm_refresh.UnitTearDown or charm_refresh.PeerRelationNotReady
+        # we consider a refresh to be in progress.
+        return not self.refresh or self.refresh.in_progress
+
     def handle_licenses(self) -> None:
         """Pull / Push licenses.
 
@@ -305,3 +314,15 @@ class OperatorProtocol(ABC, Object, ManagerStatusProtocol):
         self.workload.exec(["update-ca-certificates"])
         # Restart the service
         self.restart_charm_services(force=True)
+
+    def build_local_tls_directory(self) -> None:
+        """On Kubernetes, we need the local configuration directory.
+
+        This will store the certificates locally, which allows to construct the
+        same URIS to connect locally and on the sidecar container running
+        mongodb.
+        """
+        if self.substrate == Substrates.VM:
+            return
+
+        Path(self.state.paths.conf_path).mkdir(exist_ok=True)
