@@ -9,14 +9,17 @@ from __future__ import annotations
 import logging
 from typing import TYPE_CHECKING
 
+from botocore.exceptions import ConnectTimeoutError, SSLError
 from ops.charm import ActionEvent, RelationBrokenEvent, RelationJoinedEvent
 from ops.framework import Object
 
 from single_kernel_mongo.config.relations import ExternalRequirerRelations
 from single_kernel_mongo.config.statuses import BackupStatuses, MongoDBStatuses
 from single_kernel_mongo.exceptions import (
+    FailedToCreateS3BucketError,
     InvalidArgumentForActionError,
     InvalidPBMStatusError,
+    InvalidS3CredentialsError,
     ListBackupError,
     NonDeferrableFailedHookChecksError,
     PBMBusyError,
@@ -76,7 +79,7 @@ class BackupEventsHandler(Object):
 
     def _on_s3_relation_joined(self, event: RelationJoinedEvent) -> None:
         """Checks for valid integration for s3-integrations."""
-        if self.dependent.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             logger.warning(
                 "Adding s3-relations is not supported during an upgrade. The charm may be in a broken, unrecoverable state."
             )
@@ -87,14 +90,14 @@ class BackupEventsHandler(Object):
                 "Shard does not support S3 relations. Please relate s3-integrator to config-server only."
             )
             self.manager.state.statuses.add(
-                MongoDBStatuses.INVALID_S3_INTEGRATION_STATUS.value,
+                MongoDBStatuses.INVALID_S3_REL.value,
                 scope="unit",
                 component=self.manager.name,
             )
 
     def _on_s3_credential_changed(self, event: CredentialsChangedEvent) -> None:  # noqa: C901
         action = "configure-pbm"
-        if self.dependent.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             logger.warning(
                 "Changing s3-credentials is not supported during an upgrade. The charm may be in a broken, unrecoverable state."
             )
@@ -105,7 +108,7 @@ class BackupEventsHandler(Object):
                 "Shard does not support s3 relations, please relate s3-integrator to config-server only."
             )
             self.manager.state.statuses.add(
-                MongoDBStatuses.INVALID_S3_INTEGRATION_STATUS.value,
+                MongoDBStatuses.INVALID_S3_REL.value,
                 scope="unit",
                 component=self.manager.name,
             )
@@ -119,14 +122,49 @@ class BackupEventsHandler(Object):
             )
             return
 
+        if not self.manager.validate_s3_config():
+            logger.warning(
+                "Relation to S3 charm exists but not all necessary configurations have been set."
+            )
+            self.manager.state.statuses.set(
+                BackupStatuses.PBM_MISSING_CONF.value,
+                scope="unit",
+                component=self.manager.name,
+            )
+            return
+
         # Get the credentials from S3 connection
         credentials = self.s3_client.get_s3_connection_info()
 
         try:
+            # We can clear all statuses as they will get rewritten right after if needed.
+            # The only ones we don't want to lose were checked earlier and returned early.
+            self.manager.state.statuses.clear(
+                scope="unit",
+                component=self.manager.name,
+            )
+            # First create the bucket if it does not exist.
+            self.manager.create_bucket(credentials=credentials)
+            # Then set the config options on PBM.
             self.manager.set_config_options(credentials=credentials)
+            # Finally, resync the configuration.
             self.manager.resync_config_options()
+        except InvalidS3CredentialsError:
+            logger.error("Incorrect S3 credentials")
+            self.manager.state.statuses.add(
+                BackupStatuses.PBM_INCORRECT_CREDS.value, scope="unit", component=self.manager.name
+            )
+            return
+        except (FailedToCreateS3BucketError, SSLError, ConnectTimeoutError) as err:
+            logger.error("Failed to create bucket: %s", err)
+            self.manager.state.statuses.add(
+                BackupStatuses.FAILED_TO_CREATE_BUCKET.value,
+                scope="unit",
+                component=self.manager.name,
+            )
+            event.defer()
+            return
         except SetPBMConfigError:
-            logger.error("Failed to configure s3 backup options")
             self.manager.state.statuses.add(
                 BackupStatuses.CANT_CONFIGURE.value, scope="unit", component=self.manager.name
             )
@@ -175,6 +213,7 @@ class BackupEventsHandler(Object):
     def _on_s3_relation_broken(self, event: RelationBrokenEvent) -> None:
         """Proceed on s3 relation broken."""
         self.manager.cleanup_certs_and_restart(event.relation)
+        self.manager.state.statuses.clear(scope="unit", component=self.manager.name)
 
     def _on_create_backup_action(self, event: ActionEvent) -> None:
         action = "backup"
@@ -184,8 +223,14 @@ class BackupEventsHandler(Object):
             )
             return
 
+        # Get the credentials from S3 connection
+        credentials = self.s3_client.get_s3_connection_info()
+
         try:
             self.assert_pass_sanity_checks()
+
+            self.manager.create_bucket(credentials=credentials)
+
             self.manager.assert_can_backup()
             backup_id = self.manager.create_backup_action()
             self.charm.status_handler.set_running_status(
@@ -204,6 +249,10 @@ class BackupEventsHandler(Object):
         except (
             NonDeferrableFailedHookChecksError,
             InvalidPBMStatusError,
+            FailedToCreateS3BucketError,
+            InvalidS3CredentialsError,
+            SSLError,
+            ConnectTimeoutError,
             Exception,
         ) as e:
             fail_action_with_error_log(logger, event, action, str(e))
@@ -236,7 +285,7 @@ class BackupEventsHandler(Object):
             )
             return
 
-        if self.dependent.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             fail_action_with_error_log(
                 logger,
                 event,
