@@ -14,6 +14,7 @@ from ops.framework import Object
 
 from single_kernel_mongo.config.statuses import MongosStatuses
 from single_kernel_mongo.exceptions import (
+    DatabaseRequestedHasNotRunYetError,
     DeferrableError,
     DeferrableFailedHookChecksError,
     NonDeferrableFailedHookChecksError,
@@ -43,7 +44,7 @@ class ClusterConfigServerEventHandler(Object):
         self.charm = self.dependent.charm
         self.manager = self.dependent.cluster_manager
         self.relation_name = self.manager.relation_name
-        super().__init__(parent=self.manager, key=dependent.cluster_manager.relation_name)
+        super().__init__(parent=self.manager, key=dependent.cluster_manager.relation_name.value)
 
         self.database_provider_events = DatabaseProviderEventHandlers(
             self.charm, self.manager.data_interface
@@ -54,14 +55,14 @@ class ClusterConfigServerEventHandler(Object):
             self._on_database_requested,
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_changed, self._on_relation_event
+            self.charm.on[self.relation_name.value].relation_changed, self._on_relation_event
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_departed,
+            self.charm.on[self.relation_name.value].relation_departed,
             self.dependent.check_relation_broken_or_scale_down,
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_broken,
+            self.charm.on[self.relation_name.value].relation_broken,
             self._on_relation_broken_event,
         )
 
@@ -71,7 +72,7 @@ class ClusterConfigServerEventHandler(Object):
         Calls the manager to share the secrets with mongos charm.
         """
         try:
-            self.manager.share_secret_to_mongos(event.relation)
+            self.manager.share_secret_to_mongos(event.relation, initial_event=True)
         except DeferrableFailedHookChecksError as e:
             logger.info("Skipping database requested event: hook checks did not pass.")
             defer_event_with_info_log(logger, event, str(type(event)), str(e))
@@ -95,6 +96,8 @@ class ClusterConfigServerEventHandler(Object):
             defer_event_with_info_log(logger, event, str(type(event)), str(e))
         except NonDeferrableFailedHookChecksError as e:
             logger.info(f"Skipping {str(type(event))}: {str(e)}")
+        except DatabaseRequestedHasNotRunYetError:
+            logger.info("Not cleaning users, relation was not established yet.")
 
 
 class ClusterMongosEventHandler(Object):
@@ -112,22 +115,22 @@ class ClusterMongosEventHandler(Object):
         )
 
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_created,
+            self.charm.on[self.relation_name.value].relation_created,
             self._on_relation_created,
         )
         self.framework.observe(
             self.database_requirer_events.on.database_created, self._on_database_created
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_changed,
+            self.charm.on[self.relation_name.value].relation_changed,
             self._on_relation_changed,
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_departed,
+            self.charm.on[self.relation_name.value].relation_departed,
             self.dependent.check_relation_broken_or_scale_down,
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_broken, self._on_relation_broken
+            self.charm.on[self.relation_name.value].relation_broken, self._on_relation_broken
         )
 
     def _on_relation_created(self, event: RelationCreatedEvent) -> None:
