@@ -51,13 +51,18 @@ from single_kernel_mongo.config.statuses import (
 from single_kernel_mongo.core.operator import OperatorProtocol
 from single_kernel_mongo.exceptions import (
     ContainerNotReadyError,
+    DeferrableFailedHookChecksError,
+    InvalidConfigRoleError,
     InvalidLdapQueryTemplateError,
     InvalidLdapUserToDnMappingError,
+    NonDeferrableFailedHookChecksError,
+    SetPasswordError,
     UpgradeInProgressError,
     WaitingForLeaderError,
     WorkloadNotReadyError,
     WorkloadServiceError,
 )
+from single_kernel_mongo.utils.event_helpers import defer_event_with_info_log
 from single_kernel_mongo.utils.mongo_connection import NotReadyError
 
 logger = logging.getLogger(__name__)
@@ -102,11 +107,19 @@ class LifecycleEventsHandler(Object):
 
         if self.dependent.name == CharmKind.MONGOD:
             self.framework.observe(
-                getattr(self.charm.on, "mongodb_storage_attached"),
+                getattr(self.charm.on, "archive_storage_attached"),
                 self.on_storage_attached,
             )
             self.framework.observe(
-                getattr(self.charm.on, "mongodb_storage_detaching"),
+                getattr(self.charm.on, "data_storage_attached"),
+                self.on_storage_attached,
+            )
+            self.framework.observe(
+                getattr(self.charm.on, "logs_storage_attached"),
+                self.on_storage_attached,
+            )
+            self.framework.observe(
+                getattr(self.charm.on, "data_storage_detaching"),
                 self.on_storage_detaching,
             )
 
@@ -117,8 +130,13 @@ class LifecycleEventsHandler(Object):
         """Start event."""
         try:
             self.dependent.prepare_for_startup()
-        except (ContainerNotReadyError, WorkloadServiceError):
-            logger.info("Not ready to start.")
+        except (ContainerNotReadyError, WorkloadServiceError) as e:
+            defer_event_with_info_log(
+                logger, event, "start", f"Not ready to start: {e.__class__.__name__}({e})"
+            )
+            return
+        except InvalidConfigRoleError:
+            logger.info("Missing a valid role.")
             event.defer()
             return
         except WorkloadNotReadyError:
@@ -161,8 +179,23 @@ class LifecycleEventsHandler(Object):
         """Config Changed Event."""
         try:
             self.dependent.update_config_and_restart()
-        except (UpgradeInProgressError, WaitingForLeaderError):
+        except (
+            UpgradeInProgressError,
+            WaitingForLeaderError,
+            DeferrableFailedHookChecksError,
+            SetPasswordError,
+        ) as e:
+            defer_event_with_info_log(logger, event, str(type(event)), str(e))
+        except InvalidConfigRoleError:
+            logger.info("Invalid config role.")
+            if self.charm.unit.is_leader():
+                self.dependent.state.statuses.add(
+                    MongoDBStatuses.INVALID_ROLE.value,
+                    scope="app",
+                    component=self.dependent.name,
+                )
             event.defer()
+            return
         except InvalidLdapUserToDnMappingError:
             self.dependent.state.statuses.add(
                 LdapStatuses.INVALID_LDAP_USER_MAPPING.value,
@@ -175,6 +208,8 @@ class LifecycleEventsHandler(Object):
                 scope="unit",
                 component=self.dependent.name,
             )
+        except (NonDeferrableFailedHookChecksError, WorkloadServiceError):
+            logger.info(f"{str(type(event))} will be skipped.")
 
     def on_update_status(self, event: UpdateStatusEvent):
         """Update Status Event."""
@@ -188,7 +223,7 @@ class LifecycleEventsHandler(Object):
                 secret_id=event.secret.id or "",
             )
         except (WorkloadServiceError, ChangeError) as err:
-            logger.info("Failed to restart services", err, exc_info=True)
+            logger.info("Failed to restart services: %s", err, exc_info=True)
             self.dependent.state.statuses.add(
                 CharmStatuses.FAILED_SERVICES_START.value,
                 scope="unit",
@@ -196,6 +231,10 @@ class LifecycleEventsHandler(Object):
             )
             event.defer()
             return
+        except (DeferrableFailedHookChecksError, SetPasswordError) as e:
+            defer_event_with_info_log(logger, event, str(type(event)), str(e))
+        except NonDeferrableFailedHookChecksError as e:
+            logger.info(f"{str(type(event))} will be skipped: {str(e)}")
 
     def on_relation_joined(self, event: RelationJoinedEvent):
         """Relation joined event."""
