@@ -19,6 +19,7 @@ from ops.framework import Object
 from pymongo.errors import OperationFailure, PyMongoError, ServerSelectionTimeoutError
 
 from single_kernel_mongo.config.literals import TrustStoreFiles
+from single_kernel_mongo.config.statuses import ShardStatuses
 from single_kernel_mongo.exceptions import (
     BalancerNotEnabledError,
     DeferrableFailedHookChecksError,
@@ -53,7 +54,9 @@ class ConfigServerEventHandler(Object):
         self.charm = self.dependent.charm
         self.manager = self.dependent.config_server_manager
         self.relation_name = self.manager.relation_name
-        super().__init__(parent=self.manager, key=dependent.config_server_manager.relation_name)
+        super().__init__(
+            parent=self.manager, key=dependent.config_server_manager.relation_name.value
+        )
 
         self.database_provider_events = DatabaseProviderEventHandlers(
             self.charm, self.manager.data_interface
@@ -62,14 +65,14 @@ class ConfigServerEventHandler(Object):
             self.database_provider_events.on.database_requested, self._on_database_requested
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_departed,
+            self.charm.on[self.relation_name.value].relation_departed,
             self.dependent.check_relation_broken_or_scale_down,
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_changed, self._on_relation_event
+            self.charm.on[self.relation_name.value].relation_changed, self._on_relation_event
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_broken, self._on_relation_event
+            self.charm.on[self.relation_name.value].relation_broken, self._on_relation_event
         )
 
     def _on_relation_event(self, event: RelationChangedEvent):
@@ -117,10 +120,10 @@ class ShardEventHandler(Object):
         )
 
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_created, self._on_relation_created
+            self.charm.on[self.relation_name.value].relation_created, self._on_relation_created
         )
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_changed, self._on_database_created
+            self.charm.on[self.relation_name.value].relation_changed, self._on_database_created
         )
 
         self.framework.observe(
@@ -128,12 +131,12 @@ class ShardEventHandler(Object):
         )
 
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_departed,
+            self.charm.on[self.relation_name.value].relation_departed,
             self.dependent.check_relation_broken_or_scale_down,
         )
 
         self.framework.observe(
-            self.charm.on[self.relation_name].relation_broken, self._on_relation_broken
+            self.charm.on[self.relation_name.value].relation_broken, self._on_relation_broken
         )
 
     def _on_relation_created(self, event: RelationCreatedEvent):
@@ -172,5 +175,10 @@ class ShardEventHandler(Object):
         except DeferrableFailedHookChecksError as e:
             defer_event_with_info_log(logger, event, str(type(event)), str(e))
         except NonDeferrableFailedHookChecksError as e:
+            self.manager.state.statuses.set(
+                ShardStatuses.MISSING_CONF_SERVER_REL.value,
+                scope="unit",
+                component=self.manager.name,
+            )
             self.dependent.remove_ca_cert_from_trust_store(TrustStoreFiles.PBM)
             logger.info(f"Skipping {str(type(event))}: {str(e)}")
