@@ -4,10 +4,6 @@
 
 """All general exceptions."""
 
-from data_platform_helpers.advanced_statuses.models import StatusObject
-
-from single_kernel_mongo.config.statuses import UpgradeStatuses
-
 
 class InvalidCharmKindError(Exception):
     """Raised when calling a function on the wrong charm kind."""
@@ -82,6 +78,14 @@ class ListBackupError(Exception):
     """Raised when list backup operation is failed."""
 
 
+class InvalidS3CredentialsError(Exception):
+    """Raised when list backup operation is failed."""
+
+
+class FailedToCreateS3BucketError(Exception):
+    """Raised when list backup operation is failed."""
+
+
 class FailedToFindNodePortError(Exception):
     """Raised when NodePort cannot be found, but is excepted to be present."""
 
@@ -114,6 +118,10 @@ class UpgradeInProgressError(Exception):
     """Raised when an upgrade is in progress."""
 
 
+class MongoDBUpgradeError(Exception):
+    """Raised when the snap upgrade fails."""
+
+
 class OpenPortFailedError(Exception):
     """Raised when we fail to open ports."""
 
@@ -201,22 +209,6 @@ class EarlyRemovalOfConfigServerError(Exception):
     """Raised when we try to remove config server while it still has shards."""
 
 
-class StatusError(Exception):
-    """Exception with ops status."""
-
-    def __init__(self, status: StatusObject) -> None:
-        super().__init__(status.message)
-        self.status = status
-
-
-class PrecheckFailedError(StatusError):
-    """App is not ready to upgrade."""
-
-    def __init__(self, message: str):
-        self.message = message
-        super().__init__(UpgradeStatuses.REFRESH_IN_PROGRESS.value)
-
-
 class FailedToElectNewPrimaryError(Exception):
     """Raised when a new primary isn't elected after stepping down."""
 
@@ -241,10 +233,6 @@ class ActionFailedError(Exception):
     """Raised when we failed an action."""
 
 
-class UnhealthyUpgradeError(Exception):
-    """Raised when the upgrade is unhealthy during an post upgrade check."""
-
-
 class WaitingForLdapDataError(DeferrableError):
     """Raised when the charm hasn't received data from ldap yet."""
 
@@ -275,3 +263,11 @@ class MissingCredentialsError(Exception):
 
 class InvalidLdapHashError(DeferrableError):
     """Raised when the hash shared by config server is invalid."""
+
+
+class InvalidPasswordError(Exception):
+    """Raised when an invalid password is found in the system-users config secret."""
+
+
+class InvalidConfigRoleError(Exception):
+    """Raised when the role is invalid."""
