@@ -51,7 +51,7 @@ class ClusterProvider(Object):
         substrate: Substrates,
         relation_name: RelationNames = RelationNames.CLUSTER,
     ):
-        super().__init__(parent=dependent, key=relation_name)
+        super().__init__(parent=dependent, key=relation_name.value)
         self.dependent = dependent
         self.charm = dependent.charm
         self.state = state
@@ -59,14 +59,14 @@ class ClusterProvider(Object):
         self.relation_name = relation_name
         self.data_interface = self.state.cluster_provider_data_interface
 
-    def assert_pass_hook_checks(self) -> None:
+    def assert_pass_hook_checks(self, initial_event: bool = False) -> None:
         """Runs the pre hook checks, raises if it fails."""
         if not self.state.db_initialised:
             raise DeferrableFailedHookChecksError("DB is not initialised")
 
         if not self.is_valid_mongos_integration():
             self.state.statuses.add(
-                MongoDBStatuses.UNSUPPORTED_MONGOS_REL.value,
+                MongoDBStatuses.INVALID_MONGOS_REL.value,
                 scope="unit",
                 component=self.dependent.name,
             )
@@ -77,7 +77,7 @@ class ClusterProvider(Object):
         if not self.charm.unit.is_leader():
             raise NonDeferrableFailedHookChecksError("Not leader")
 
-        if self.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress and initial_event:
             raise DeferrableFailedHookChecksError(
                 "Processing mongos applications is not supported during an upgrade. The charm may be in a broken, unrecoverable state."
             )
@@ -88,12 +88,12 @@ class ClusterProvider(Object):
         # we don't have any cluster relation.
         return self.state.is_role(MongoDBRoles.CONFIG_SERVER) or not self.state.cluster_relations
 
-    def share_secret_to_mongos(self, relation: Relation) -> None:
+    def share_secret_to_mongos(self, relation: Relation, initial_event: bool = False) -> None:
         """Handles the database requested event.
 
         The first time secrets are written to relations should be on this event.
         """
-        self.assert_pass_hook_checks()
+        self.assert_pass_hook_checks(initial_event=initial_event)
 
         config_server_db = self.state.generate_config_server_db()
         self.dependent.mongo_manager.reconcile_mongo_users_and_dbs(relation)
@@ -133,7 +133,7 @@ class ClusterProvider(Object):
         If it has departed, we run some checks and if we are a VM charm, we
         proceed to reconcile the users and DB and cleanup mongoDB.
         """
-        if self.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             logger.warning(
                 "Removing integration to mongos is not supported during an upgrade. The charm may be in a broken, unrecoverable state."
             )
@@ -158,6 +158,9 @@ class ClusterProvider(Object):
 
         config_server_db = self.state.generate_config_server_db()
         for relation in self.state.cluster_relations:
+            if not self.data_interface.fetch_relation_field(relation.id, "database"):
+                logger.info("Database Requested has not run yet, skipping.")
+                continue
             self.data_interface.update_relation_data(
                 relation.id,
                 {
@@ -177,6 +180,9 @@ class ClusterProvider(Object):
             return
 
         for relation in self.state.cluster_relations:
+            if not self.data_interface.fetch_relation_field(relation.id, "database"):
+                logger.info("Database Requested has not run yet, skipping.")
+                continue
             self.data_interface.update_relation_data(
                 relation.id,
                 {ClusterStateKeys.LDAP_HASH.value: hashed_data},
@@ -194,6 +200,9 @@ class ClusterProvider(Object):
             return
 
         for relation in self.state.cluster_relations:
+            if not self.data_interface.fetch_relation_field(relation.id, "database"):
+                logger.info("Database Requested has not run yet, skipping.")
+                continue
             self.data_interface.delete_relation_data(
                 relation.id,
                 [ClusterStateKeys.LDAP_HASH.value],
@@ -211,6 +220,9 @@ class ClusterProvider(Object):
             return
 
         for relation in self.state.cluster_relations:
+            if not self.data_interface.fetch_relation_field(relation.id, "database"):
+                logger.info("Database Requested has not run yet, skipping.")
+                continue
             self.data_interface.update_relation_data(
                 relation.id,
                 {
@@ -230,7 +242,7 @@ class ClusterRequirer(Object):
         substrate: Substrates,
         relation_name: RelationNames = RelationNames.CLUSTER,
     ):
-        super().__init__(parent=dependent, key=relation_name)
+        super().__init__(parent=dependent, key=relation_name.value)
         self.dependent = dependent
         self.charm = dependent.charm
         self.state = state
@@ -257,8 +269,8 @@ class ClusterRequirer(Object):
             raise DeferrableFailedHookChecksError(
                 "Mongos was waiting for config-server to enable TLS. Wait for TLS to be enabled until starting mongos."
             )
-        if self.state.upgrade_in_progress:
-            raise DeferrableFailedHookChecksError(
+        if self.dependent.refresh_in_progress:
+            logger.warning(
                 "Processing client applications is not supported during an upgrade. The charm may be in a broken, unrecoverable state."
             )
 
@@ -278,7 +290,7 @@ class ClusterRequirer(Object):
         """
         if not username or not password:
             raise WaitingForSecretsError
-        if self.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             logger.warning(
                 "Processing client applications is not supported during an upgrade. The charm may be in a broken, unrecoverable state."
             )
@@ -321,7 +333,7 @@ class ClusterRequirer(Object):
             if not self.dependent.is_mongos_running():
                 logger.info("Mongos has not started yet, deferring")
                 self.state.statuses.set(
-                    MongosStatuses.MONGOS_NOT_STARTED.value,
+                    MongosStatuses.WAITING_FOR_MONGOS_START.value,
                     scope="unit",
                     component=self.dependent.name,
                 )
@@ -457,9 +469,9 @@ class ClusterRequirer(Object):
         mongos_has_tls, config_server_has_tls = self.tls_status()
         match (mongos_has_tls, config_server_has_tls):
             case False, True:
-                return MongosStatuses.REQUIRES_TLS.value
+                return MongosStatuses.MISSING_TLS_REL.value
             case True, False:
-                return MongosStatuses.REQUIRES_NO_TLS.value
+                return MongosStatuses.INVALID_TLS_REL.value
             case _:
                 pass
         if not self.is_ca_compatible():
