@@ -18,7 +18,6 @@ from typing_extensions import override
 from yaml import safe_dump, safe_load
 
 from single_kernel_mongo.config.literals import (
-    LOCALHOST,
     PBM_RESTART_DELAY,
     CharmKind,
     MongoPorts,
@@ -343,6 +342,9 @@ class MongoConfigManager(FileBasedConfigManager, ABC):
                             "allowInvalidCertificates": True,
                             "clusterCAFile": f"{self.workload.paths.int_ca_file}",
                             "clusterFile": f"{self.workload.paths.int_pem_file}",
+                            "clusterAuthX509": {
+                                "attributes": f"O={self.state.get_subject_name()}",
+                            },
                         }
                     },
                 },
@@ -366,7 +368,7 @@ class MongoConfigManager(FileBasedConfigManager, ABC):
                     "tls": {
                         "CAFile": f"{self.workload.paths.ext_ca_file}",
                         "certificateKeyFile": f"{self.workload.paths.ext_pem_file}",
-                        "mode": "preferTLS",
+                        "mode": "requireTLS",
                         "disabledProtocols": "TLS1_0,TLS1_1",
                     }
                 },
@@ -494,7 +496,7 @@ class MongosConfigManager(MongoConfigManager):
             return {"sharding": {"configDB": uri}}
         return {
             "sharding": {
-                "configDB": f"{self.state.app_peer_data.replica_set}/{LOCALHOST}:{MongoPorts.MONGODB_PORT}"
+                "configDB": f"{self.state.app_peer_data.replica_set}/{self.state.unit_peer_data.internal_address}:{MongoPorts.MONGODB_PORT.value}"
             }
         }
 
