@@ -14,6 +14,7 @@ from __future__ import annotations
 import json
 import logging
 from typing import TYPE_CHECKING
+from urllib.parse import urlencode
 
 from dacite import from_dict
 from data_platform_helpers.advanced_statuses.models import StatusObject
@@ -27,6 +28,7 @@ from pymongo.errors import (
     PyMongoError,
     ServerSelectionTimeoutError,
 )
+from tenacity import Retrying, stop_after_attempt, wait_fixed
 
 from single_kernel_mongo.config.literals import MongoPorts, Substrates
 from single_kernel_mongo.config.statuses import CharmStatuses, MongodStatuses
@@ -34,6 +36,7 @@ from single_kernel_mongo.core.structured_config import MongoDBRoles
 from single_kernel_mongo.exceptions import (
     DatabaseRequestedHasNotRunYetError,
     DeployedWithoutTrustError,
+    FailedToElectNewPrimaryError,
     MissingCredentialsError,
     SetPasswordError,
 )
@@ -97,14 +100,20 @@ class MongoManager(Object, ManagerStatusProtocol):
         Pass direct=True, when checking if a *single replica* is ready.
         Pass direct=False, when checking if the entire replica set is ready
         """
-        if not uri and self.state.is_role(MongoDBRoles.MONGOS):
-            uri = f"localhost:{MongoPorts.MONGOS_PORT}"
-        actual_uri = uri or "localhost"
+        port = (
+            MongoPorts.MONGOS_PORT.value
+            if self.state.is_role(MongoDBRoles.MONGOS)
+            else MongoPorts.MONGODB_PORT.value
+        )
+        params = self.state.operator_config.tls_config
+
+        actual_uri = uri or f"mongodb://localhost:{port}"
+        actual_uri = f"{actual_uri}/?{urlencode(params)}"
         with MongoConnection(EMPTY_CONFIGURATION, actual_uri, direct=direct) as direct_mongo:
             return direct_mongo.is_ready
 
-    def set_user_password(self, user: MongoDBUser, password: str) -> str:
-        """Sets the password for a given username and return the secret id.
+    def set_user_password(self, user: MongoDBUser, password: str) -> None:
+        """Sets the password for a given username in the workload and secrets.
 
         Raises:
             SetPasswordError
@@ -119,7 +128,7 @@ class MongoManager(Object, ManagerStatusProtocol):
             except PyMongoError as e:
                 raise SetPasswordError(f"Failed changing the password: {e}")
 
-        return self.state.set_user_password(user, password)
+        self.state.set_user_password(user, password)
 
     def initialise_replica_set(self) -> None:
         """Initialises the replica set."""
@@ -205,7 +214,6 @@ class MongoManager(Object, ManagerStatusProtocol):
         self.update_user(relation)
         if relation_departing:
             self.remove_user(relation)
-            self.auto_delete_db(relation)
         if relation_changed:
             self.update_diff(relation)
 
@@ -266,7 +274,7 @@ class MongoManager(Object, ManagerStatusProtocol):
                 return
 
             data_interface.set_endpoints(relation.id, ",".join(sorted(config.hosts)))
-            data_interface.set_uris(relation.id, config.uri)
+            data_interface.set_uris(relation.id, config.uri_without_tls)
 
             if not self.state.is_role(MongoDBRoles.MONGOS):
                 data_interface.set_replset(
@@ -274,7 +282,7 @@ class MongoManager(Object, ManagerStatusProtocol):
                 )
 
     def update_user(self, relation: Relation) -> None:
-        """Add the user for this relation."""
+        """Update the user for this relation."""
         data_interface = DatabaseProviderData(
             self.model,
             relation.name,
@@ -282,10 +290,14 @@ class MongoManager(Object, ManagerStatusProtocol):
 
         username = f"relation-{relation.id}"
         password = data_interface.fetch_my_relation_field(relation.id, "password")
+
         with MongoConnection(self.state.mongo_config) as mongo:
             has_user = mongo.user_exists(username)
-        if has_user:
+
+        # Don't update a user that doesn't exist
+        if not has_user:
             return
+
         with MongoConnection(self.state.mongo_config) as mongo:
             config = self.get_config(
                 username,
@@ -387,10 +399,10 @@ class MongoManager(Object, ManagerStatusProtocol):
                 relation.id,
                 ",".join(sorted(config.hosts)),
             )
-        if config.uri != uris:
+        if config.uri_without_tls != uris:
             data_interface.set_uris(
                 relation.id,
-                config.uri,
+                config.uri_without_tls,
             )
         if config.database != database:
             data_interface.set_database(
@@ -398,20 +410,6 @@ class MongoManager(Object, ManagerStatusProtocol):
                 config.database,
             )
 
-    def auto_delete_db(self, relation: Relation) -> None:
-        """Delete a DB if necessary."""
-        with MongoConnection(self.state.mongo_config) as mongo:
-            if not self.state.config.auto_delete:
-                return
-            data_interface = DatabaseProviderData(self.model, relation.name)
-            database = data_interface.fetch_relation_field(relation.id, "database")
-            if not database:  # Early return, no database to delete.
-                return
-            if database not in mongo.get_databases():  # Early return, database not in mongodb
-                return
-            logger.info(f"Drop database: {database}")
-            mongo.drop_database(database)
-
     def get_config(
         self,
         username: str,
@@ -432,8 +430,7 @@ class MongoManager(Object, ManagerStatusProtocol):
             "password": password,
             "hosts": self.state.app_hosts,
             "roles": set(roles.split(",")),
-            "tls_external": False,
-            "tls_internal": False,
+            "tls_enabled": False,
             "port": self.state.host_port,
         }
         if not self.state.is_role(MongoDBRoles.MONGOS):
@@ -477,7 +474,7 @@ class MongoManager(Object, ManagerStatusProtocol):
 
             for member in config_hosts - replset_members:
                 logger.debug("Adding %s to replica set", member)
-                if not self.mongod_ready(uri=member):
+                if not self.mongod_ready(uri=f"mongodb://{member}"):
                     logger.debug("not reconfiguring: %s is not ready yet.", member)
                     raise NotReadyError
                 mongo.add_replset_member(member)
@@ -517,15 +514,15 @@ class MongoManager(Object, ManagerStatusProtocol):
 
             match replica_status:
                 case "":
-                    return [MongodStatuses.MEMBER_BEING_ADDED.value]
+                    return [MongodStatuses.ADDING_MEMBER.value]
                 case "PRIMARY":
                     charm_statuses.append(MongodStatuses.PRIMARY.value)
                 case "SECONDARY":
                     charm_statuses.append(MongodStatuses.SECONDARY.value)
                 case "STARTUP" | "STARTUP2" | "ROLLBACK" | "RECOVERING":
-                    return [MongodStatuses.MEMBER_SYNCING.value]
+                    return [MongodStatuses.SYNCING_MEMBER.value]
                 case "REMOVED":
-                    return [MongodStatuses.MEMBER_REMOVING.value]
+                    return [MongodStatuses.REMOVING_MEMBER.value]
                 case _:
                     return [MongodStatuses.replset_status(replica_status)]
 
@@ -540,13 +537,13 @@ class MongoManager(Object, ManagerStatusProtocol):
             # AutoReconnect is raised when a connection to the database is lost and an attempt to
             # auto-reconnect will be made by pymongo.
             logger.debug("Got error: %s, while checking replica set status", str(e))
-            return [MongodStatuses.WAITING_RECONNECT.value]
+            return [MongodStatuses.WAITING_RECONNECTION.value]
         except OperationFailure as e:
             logger.warning("Authentication Failed: %s", e, exc_info=True)
             return [MongodStatuses.WAITING_RECONFIG.value]
         except MissingCredentialsError as e:
             logger.warning("Missing credentials: %s", e, exc_info=True)
-            return [MongodStatuses.MISSING_CREDENTIALS]
+            return [MongodStatuses.WAITING_CREDENTIALS.value]
 
     def get_leader_statuses(self) -> list[StatusObject]:
         """Returns statuses that juju leader can retrieve for mongo."""
@@ -566,3 +563,34 @@ class MongoManager(Object, ManagerStatusProtocol):
             return [MongodStatuses.WAITING_RECONFIG.value]
 
         return charm_statuses
+
+    def set_feature_compatibility_version(self, feature_version: str) -> None:
+        """Sets the mongos feature compatibility version."""
+        if self.state.is_role(MongoDBRoles.REPLICATION):
+            config = self.state.mongo_config
+        elif self.state.is_role(MongoDBRoles.CONFIG_SERVER):
+            config = self.state.mongos_config
+        else:
+            return
+        with MongoConnection(config) as mongos:
+            mongos.client.admin.command(
+                "setFeatureCompatibilityVersion", value=feature_version, confirm=True
+            )
+
+    def step_down_primary_and_wait_reelection(self):
+        """Steps down the current primary and waits for a new one to be elected."""
+        if len(self.state.internal_hosts) < 2:
+            logger.warning(
+                "No secondaries to become primary - upgrading primary without electing a new one, expect downtime."
+            )
+            return
+
+        old_primary = self.dependent.primary_unit_name  # type: ignore
+        with MongoConnection(self.state.mongo_config) as mongod:
+            mongod.step_down_primary()
+
+        for attempt in Retrying(stop=stop_after_attempt(30), wait=wait_fixed(1), reraise=True):
+            with attempt:
+                new_primary = self.dependent.primary_unit_name  # type: ignore
+                if new_primary == old_primary:
+                    raise FailedToElectNewPrimaryError()
