@@ -180,9 +180,12 @@ class ConfigServerManager(Object, ManagerStatusProtocol):
         Raises:
             NonDeferrableFailedHookChecksError, DeferrableFailedHookChecksError
         """
+        if not self.state.is_role(MongoDBRoles.CONFIG_SERVER):
+            raise NonDeferrableFailedHookChecksError("is only executed by config-server")
         if not self.state.db_initialised:
             raise DeferrableFailedHookChecksError("db is not initialised.")
-        if not self.dependent.is_relation_feasible(self.relation_name):
+        if status := self.dependent.get_relation_feasible_status(self.relation_name):
+            self.dependent.state.statuses.add(status, scope="unit", component=self.dependent.name)
             raise NonDeferrableFailedHookChecksError("relation is not feasible")
         if not self.charm.unit.is_leader():
             raise NonDeferrableFailedHookChecksError
@@ -194,10 +197,8 @@ class ConfigServerManager(Object, ManagerStatusProtocol):
             rev_status
             := self.dependent.cluster_version_checker.get_cluster_mismatched_revision_status()
         ):
-            self.state.statuses.add(rev_status, scope="unit", component=self.dependent.name)
+            self.state.statuses.add(rev_status, scope="app", component=self.dependent.name)
             raise DeferrableFailedHookChecksError("Mismatched versions in the cluster")
-        if not self.state.is_role(MongoDBRoles.CONFIG_SERVER):
-            raise NonDeferrableFailedHookChecksError("is only executed by config-server")
 
     def assert_pass_hook_checks(self, relation: Relation, leaving: bool = False) -> None:
         """Runs pre hooks checks and raises the appropriate error if it fails.
@@ -214,7 +215,7 @@ class ConfigServerManager(Object, ManagerStatusProtocol):
                 "Cannot add/remove shards while a backup/restore is in progress."
             )
 
-        if self.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             logger.warning(
                 "Adding/Removing shards is not supported during an upgrade. The charm may be in a broken, unrecoverable state"
             )
@@ -238,6 +239,9 @@ class ConfigServerManager(Object, ManagerStatusProtocol):
     def update_mongos_hosts(self) -> None:
         """Updates the hosts for mongos on the relation data."""
         for relation in self.state.config_server_relation:
+            if self.data_interface.fetch_relation_field(relation.id, "requested-secrets") is None:
+                logger.info(f"Database Requested event has not run yet for relation {relation.id}")
+                continue
             self.data_interface.update_relation_data(
                 relation.id,
                 {
@@ -286,13 +290,15 @@ class ConfigServerManager(Object, ManagerStatusProtocol):
         if self.dependent.cluster_version_checker.get_cluster_mismatched_revision_status():
             return charm_statuses[scope]
 
-        uri = f"mongodb://{self.state.unit_peer_data.internal_address}:{MongoPorts.MONGOS_PORT}"
+        uri = (
+            f"mongodb://{self.state.unit_peer_data.internal_address}:{MongoPorts.MONGOS_PORT.value}"
+        )
         if not self.dependent.mongo_manager.mongod_ready(uri):
             charm_statuses["unit"].append(ConfigServerStatuses.MONGOS_NOT_RUNNING.value)
 
         if not self.state.config_server_relation:
-            charm_statuses["unit"].append(ConfigServerStatuses.MISSING_SHARDING_REL.value)
-            charm_statuses["app"].append(ConfigServerStatuses.MISSING_SHARDING_REL.value)
+            charm_statuses["unit"].append(ConfigServerStatuses.MISSING_CONF_SERVER_REL.value)
+            charm_statuses["app"].append(ConfigServerStatuses.MISSING_CONF_SERVER_REL.value)
             # return as other statuses require shard(s) to compute
             return charm_statuses[scope]
 
@@ -344,7 +350,7 @@ class ConfigServerManager(Object, ManagerStatusProtocol):
             return
 
         self.state.statuses.delete(
-            ConfigServerStatuses.MISSING_SHARDING_REL.value, scope="unit", component=self.name
+            ConfigServerStatuses.MISSING_CONF_SERVER_REL.value, scope="unit", component=self.name
         )
 
         self.charm.status_handler.set_running_status(
@@ -501,11 +507,14 @@ class ShardManager(Object, ManagerStatusProtocol):
 
     def assert_pass_sanity_hook_checks(self, is_leaving: bool) -> None:
         """Returns True if all the sanity hook checks for sharding pass."""
+        if not self.state.is_role(MongoDBRoles.SHARD):
+            raise NonDeferrableFailedHookChecksError("is only executed by shards")
         if not self.state.db_initialised:
             raise DeferrableFailedHookChecksError("db is not initialised.")
-        if not self.dependent.is_relation_feasible(self.relation_name):
+        if (status := self.dependent.get_relation_feasible_status(self.relation_name)) is not None:
+            self.dependent.state.statuses.add(status, scope="unit", component=self.dependent.name)
             raise NonDeferrableFailedHookChecksError("relation is not feasible")
-        if self.state.upgrade_in_progress:
+        if self.dependent.refresh_in_progress:
             logger.warning(
                 "Adding/Removing shards is not supported during an upgrade. The charm may be in a broken, unrecoverable state"
             )
@@ -519,10 +528,8 @@ class ShardManager(Object, ManagerStatusProtocol):
             rev_status
             := self.dependent.cluster_version_checker.get_cluster_mismatched_revision_status()
         ):
-            self.state.statuses.add(rev_status, scope="unit", component=self.dependent.name)
+            self.state.statuses.add(rev_status, scope="app", component=self.dependent.name)
             raise DeferrableFailedHookChecksError("Mismatched versions in the cluster")
-        if not self.state.is_role(MongoDBRoles.SHARD):
-            raise NonDeferrableFailedHookChecksError("is only executed by shards")
 
     def assert_pass_hook_checks(self, relation: Relation, is_leaving: bool = False) -> None:
         """Runs the pre-hooks checks, returns True if all pass."""
@@ -533,11 +540,7 @@ class ShardManager(Object, ManagerStatusProtocol):
         if relation.app is None:
             raise NonDeferrableFailedHookChecksError("Missing app information in event, skipping.")
 
-        # We look in the relation databag because the shard state data
-        # interface returns nothing on relation broken event.
-        mongos_hosts = relation.data[relation.app].get(AppShardingComponentKeys.HOST.value)
-
-        if is_leaving and not mongos_hosts:
+        if is_leaving and not self.state.app_peer_data.mongos_hosts:
             raise NonDeferrableFailedHookChecksError(
                 "Config-server never set up, no need to process broken event."
             )
@@ -656,7 +659,7 @@ class ShardManager(Object, ManagerStatusProtocol):
             return
         # many secret changed events occur, only listen to those related to our interface with the
         # config-server
-        sharding_secretes_label = f"{self.relation_name}.{relation.id}.extra.secret"
+        sharding_secretes_label = f"{self.relation_name.value}.{relation.id}.extra.secret"
         if secret_label != sharding_secretes_label:
             logger.info(
                 f"Secret unrelated to this sharding relation {relation.id} is changing, ignoring event."
@@ -696,10 +699,13 @@ class ShardManager(Object, ManagerStatusProtocol):
         """Waits for the shard to be fully drained from the cluster."""
         self.assert_pass_hook_checks(relation, is_leaving=True)
 
-        mongos_hosts = self.state.app_peer_data.mongos_hosts
+        if not (mongos_hosts := self.state.app_peer_data.mongos_hosts):
+            return
 
         self.wait_for_draining(mongos_hosts)
 
+        self.state.app_peer_data.mongos_hosts = []
+
         self.state.statuses.set(
             ShardStatuses.SHARD_DRAINED.value, scope="unit", component=self.name
         )
