@@ -13,12 +13,16 @@ from typing import TYPE_CHECKING, TypeVar
 from urllib.parse import quote
 
 from data_platform_helpers.advanced_statuses.protocol import StatusesState, StatusesStateProtocol
-from ops import Object, Relation, Unit
-from pymongo.errors import OperationFailure, ServerSelectionTimeoutError
+from ops import ModelError, Object, Relation, SecretNotFoundError, Unit
+from pymongo.errors import (
+    AutoReconnect,
+    NotPrimaryError,
+    OperationFailure,
+    ServerSelectionTimeoutError,
+)
 
 from single_kernel_mongo.config.literals import (
     SECRETS_UNIT,
-    SNAP,
     CharmKind,
     MongoPorts,
     Scope,
@@ -62,19 +66,15 @@ from single_kernel_mongo.state.tls_state import TLSState
 from single_kernel_mongo.state.unit_peer_state import (
     UnitPeerReplicaSet,
 )
-from single_kernel_mongo.state.upgrade_state import (
-    AppUpgradePeerData,
-    UnitUpgradePeerData,
-)
 from single_kernel_mongo.utils.helpers import (
     generate_relation_departed_key,
-    unit_number,
 )
 from single_kernel_mongo.utils.mongo_config import MongoConfiguration
 from single_kernel_mongo.utils.mongo_connection import MongoConnection
 from single_kernel_mongo.utils.mongo_error_codes import MongoErrorCodes
 from single_kernel_mongo.utils.mongodb_users import (
     BackupUser,
+    InternalUsers,
     LogRotateUser,
     MongoDBUser,
     MonitorUser,
@@ -137,12 +137,6 @@ class CharmState(Object, StatusesStateProtocol):
             self.ldap_peer_relation_name,
         )
 
-        self.upgrade_app_interface = DataPeerData(
-            self.model, relation_name=RelationNames.UPGRADE_VERSION.value
-        )
-        self.upgrade_unit_interface = DataPeerUnitData(
-            self.model, relation_name=RelationNames.UPGRADE_VERSION.value
-        )
         self.paths = MongoPaths(self.charm_role)
 
         self.k8s_manager = K8sManager(
@@ -178,6 +172,13 @@ class CharmState(Object, StatusesStateProtocol):
             return set()
         return self.peer_relation.units
 
+    @property
+    def reverse_order_peer_units(self) -> list[Unit]:
+        """Units sorted in reverse order."""
+        return sorted(
+            self.peers_units, key=lambda unit: int(unit.name.split("/")[-1]), reverse=True
+        )
+
     @property
     def client_relations(self) -> set[Relation]:
         """The set of client relations.
@@ -190,11 +191,6 @@ class CharmState(Object, StatusesStateProtocol):
             return set(self.model.relations[RelationNames.MONGOS_PROXY.value])
         return set(self.model.relations[RelationNames.DATABASE.value])
 
-    @property
-    def upgrade_relation(self) -> Relation | None:
-        """The set of upgrade relations."""
-        return self.model.get_relation(RelationNames.UPGRADE_VERSION.value)
-
     @property
     def mongos_cluster_relation(self) -> Relation | None:
         """The Mongos side of the cluster relation."""
@@ -247,7 +243,6 @@ class CharmState(Object, StatusesStateProtocol):
             data_interface=self.peer_app_interface,
             component=self.model.app,
             substrate=self.substrate,
-            role=self.config.role,
             model=self.model,
         )
 
@@ -278,48 +273,6 @@ class CharmState(Object, StatusesStateProtocol):
             k8s_manager=self.k8s_manager,
         )
 
-    @property
-    def unit_upgrade_peer_data(self) -> UnitUpgradePeerData:
-        """This unit upgrade databag."""
-        return UnitUpgradePeerData(
-            relation=self.upgrade_relation,
-            data_interface=self.upgrade_unit_interface,
-            component=self.model.unit,
-            substrate=self.substrate,
-        )
-
-    @property
-    def app_upgrade_peer_data(self) -> AppUpgradePeerData:
-        """The app upgrade databag."""
-        return AppUpgradePeerData(
-            relation=self.upgrade_relation,
-            data_interface=self.upgrade_app_interface,
-            component=self.model.app,
-            substrate=self.substrate,
-        )
-
-    @property
-    def units_upgrade_peer_data(self) -> list[UnitUpgradePeerData]:
-        """Grabs all units in the current peer relation, including this unit.
-
-        Returns:
-            Sorted list of UnitUpgradePeerData in the current upgrade relation,
-            including this unit.
-        """
-        _units = []
-        for unit, data_interface in self.upgrade_units_data_interfaces.items():
-            _units.append(
-                UnitUpgradePeerData(
-                    relation=self.upgrade_relation,
-                    data_interface=data_interface,
-                    component=unit,
-                    substrate=self.substrate,
-                )
-            )
-        _units.append(self.unit_upgrade_peer_data)
-
-        return sorted(_units, key=unit_number, reverse=True)
-
     @property
     def units(self) -> set[UnitPeerReplicaSet]:
         """Grabs all units in the current peer relation, including this unit.
@@ -359,7 +312,7 @@ class CharmState(Object, StatusesStateProtocol):
         """The Requirer Data interface for the cluster relation (config-server side)."""
         return DatabaseProviderData(
             self.model,
-            RelationNames.CLUSTER,
+            RelationNames.CLUSTER.value,
         )
 
     @property
@@ -367,7 +320,7 @@ class CharmState(Object, StatusesStateProtocol):
         """The Requirer Data interface for the cluster relation (mongos side)."""
         return DatabaseRequirerData(
             self.model,
-            RelationNames.CLUSTER,
+            RelationNames.CLUSTER.value,
             database_name=self.app_peer_data.database,
             extra_user_roles=",".join(sorted(self.app_peer_data.extra_user_roles)),
             additional_secret_fields=[
@@ -427,63 +380,6 @@ class CharmState(Object, StatusesStateProtocol):
     def db_initialised(self, other: bool):
         self.app_peer_data.db_initialised = other
 
-    @property
-    def unit_workload_container_versions(self) -> dict[str, str]:
-        """{Unit name: unique identifier for unit's workload container version}.
-
-        If and only if this version changes, the workload will restart (during upgrade or
-        rollback).
-
-        On Kubernetes, the workload & charm are upgraded together
-        On machines, the charm is upgraded before the workload
-
-        VM: container version is the snap revision.
-        K8S: container version is the stateful set hash.
-
-        This identifier should be comparable to `_app_workload_container_version` to determine if
-        the unit & app are the same workload container version.
-        """
-        if self.substrate == Substrates.K8S:
-            return self.k8s_manager.list_revisions()
-        return {
-            unit.name: unit.snap_revision
-            for unit in self.units_upgrade_peer_data
-            if unit.snap_revision
-        }
-
-    @property
-    def unit_workload_container_version(self) -> str | None:
-        """Installed snap revision for this unit."""
-        if self.substrate == Substrates.K8S:
-            return self.k8s_manager.list_revisions().get(self.model.unit.name)
-        return self.unit_upgrade_peer_data.snap_revision
-
-    @unit_workload_container_version.setter
-    def unit_workload_container_version(self, value: str):
-        if self.substrate == Substrates.VM:
-            self.unit_upgrade_peer_data.snap_revision = value
-
-    @property
-    def app_workload_container_version(self) -> str:
-        """Unique identifier for the app's workload container version.
-
-        This should match the workload version in the current Juju app charm version.
-
-        This identifier should be comparable to `_unit_workload_container_versions` to determine if
-        the app & unit are the same workload container version.
-        """
-        if self.substrate == Substrates.K8S:
-            return self.k8s_manager.get_revision()
-        return SNAP.revision
-
-    @property
-    def upgrade_in_progress(self) -> bool:
-        """Is the charm in upgrade?"""
-        app_version = self.app_workload_container_version
-        unit_versions = self.unit_workload_container_versions
-        logger.debug(f"Upgrade in progress check: {app_version = } / {unit_versions = }")
-        return any(version != app_version for version in unit_versions.values())
-
     @property
     def bind_address(self) -> IPv4Address | IPv6Address | str:
         """The network binding address from the peer relation."""
@@ -502,6 +398,10 @@ class CharmState(Object, StatusesStateProtocol):
         """Sets the user password for a system user."""
         return self.secrets.set(user.password_key_name, content, Scope.APP).label
 
+    def internal_user_passwords_are_initialized(self) -> bool:
+        """Returns true if all the charmed users have a password."""
+        return all(self.get_user_password(user) for user in InternalUsers)
+
     def get_user_credentials(self) -> tuple[str | None, str | None]:
         """Retrieve the user credentials."""
         return (
@@ -532,18 +432,6 @@ class CharmState(Object, StatusesStateProtocol):
             for unit in self.peers_units
         }
 
-    @property
-    def upgrade_units_data_interfaces(self) -> dict[Unit, DataPeerOtherUnitData]:
-        """The cluster peer relation."""
-        return {
-            unit: DataPeerOtherUnitData(
-                model=self.model,
-                unit=unit,
-                relation_name=RelationNames.UPGRADE_VERSION.value,
-            )
-            for unit in self.peers_units
-        }
-
     @property
     def formatted_socket_path(self) -> str:
         """URL encoded socket path.
@@ -595,8 +483,8 @@ class CharmState(Object, StatusesStateProtocol):
         if self.is_role(MongoDBRoles.MONGOS):
             if self.is_external_client and self.substrate == Substrates.K8S:
                 return self.unit_peer_data.node_port
-            return MongoPorts.MONGOS_PORT
-        return MongoPorts.MONGODB_PORT
+            return MongoPorts.MONGOS_PORT.value
+        return MongoPorts.MONGODB_PORT.value
 
     @property
     def config_server_data_interface(self) -> DatabaseProviderData:
@@ -648,10 +536,21 @@ class CharmState(Object, StatusesStateProtocol):
         )
         return None
 
+    def get_subject_name(self) -> str:
+        """Generate the subject name for CSR."""
+        # In sharded MongoDB deployments it is a requirement that all subject names match across
+        # all cluster components. The config-server name is the source of truth across mongos and
+        # shard deployments.
+        if self.is_role(MongoDBRoles.REPLICATION) or self.is_role(MongoDBRoles.CONFIG_SERVER):
+            return self.model.app.name
+        # until integrated with config-server use current app name as
+        # subject name
+        return self.config_server_name or self.model.app.name
+
     def generate_config_server_db(self) -> str:
         """Generates the config server DB URI."""
         replica_set_name = self.model.app.name
-        hosts = sorted(f"{host}:{MongoPorts.MONGODB_PORT}" for host in self.internal_hosts)
+        hosts = sorted(f"{host}:{MongoPorts.MONGODB_PORT.value}" for host in self.internal_hosts)
         return f"{replica_set_name}/{','.join(hosts)}"
 
     # END: Helpers
@@ -700,19 +599,16 @@ class CharmState(Object, StatusesStateProtocol):
         self.unit_peer_data.update({rel_departed_key: json.dumps(scaling_down)})
         return scaling_down
 
-    @property
-    def upgrade_resumed(self) -> bool:
-        """Whether the user has resumed upgrade with juju action."""
-        if self.substrate == Substrates.K8S:
-            return self.k8s_manager.get_partition() < unit_number(self.units_upgrade_peer_data[0])
-        return self.app_upgrade_peer_data.upgrade_resumed
-
     def is_shard_added_to_cluster(self) -> bool:
         """Returns true if the shard has been added to the clusted."""
         # this information is required in order to check if we have been added
         if not self.config_server_name or not self.app_peer_data.mongos_hosts:
             return False
 
+        # We can't check if we don't have a valid certificate
+        if self.shard_state.internal_ca_secret is not None and not self.tls.external_enabled:
+            return False
+
         try:
             # check our ability to use connect to mongos
             with MongoConnection(self.remote_mongos_config) as mongos:
@@ -725,7 +621,7 @@ class CharmState(Object, StatusesStateProtocol):
             ):
                 return False
             raise
-        except ServerSelectionTimeoutError:
+        except (ServerSelectionTimeoutError, AutoReconnect, NotPrimaryError):
             # Connection refused, - this occurs when internal membership is not in sync across the
             # cluster (i.e. TLS + KeyFile).
             return False
@@ -767,10 +663,11 @@ class CharmState(Object, StatusesStateProtocol):
             username=user.username,
             password=self.get_user_password(user),
             hosts=hosts or user.hosts,
-            port=MongoPorts.MONGODB_PORT,
+            port=MongoPorts.MONGODB_PORT.value,
             roles=user.roles,
-            tls_external=self.tls.external_enabled,
-            tls_internal=self.tls.internal_enabled,
+            tls_enabled=self.tls.external_enabled,
+            tls_external_keyfile=self.paths.ext_pem_file,
+            tls_external_ca=self.paths.ext_ca_file,
             standalone=standalone,
         )
 
@@ -796,10 +693,11 @@ class CharmState(Object, StatusesStateProtocol):
             username=user.username,
             password=self.get_user_password(user),
             hosts=hosts or user.hosts,
-            port=MongoPorts.MONGOS_PORT,
+            port=MongoPorts.MONGOS_PORT.value,
             roles=user.roles,
-            tls_external=self.tls.external_enabled,
-            tls_internal=self.tls.internal_enabled,
+            tls_enabled=self.tls.external_enabled,
+            tls_external_keyfile=self.paths.ext_pem_file,
+            tls_external_ca=self.paths.ext_ca_file,
         )
 
     @property
@@ -835,7 +733,7 @@ class CharmState(Object, StatusesStateProtocol):
             return self.mongos_config_for_user(OperatorUser, self.internal_hosts)
         username, password = self.get_user_credentials()
         database = self.app_peer_data.database
-        port: MongoPorts | None = MongoPorts.MONGOS_PORT
+        port: int | None = MongoPorts.MONGOS_PORT.value
         if (
             self.charm_role.name == CharmKind.MONGOS
             and self.substrate == Substrates.VM
@@ -853,8 +751,9 @@ class CharmState(Object, StatusesStateProtocol):
             # unlike the vm mongos charm, the K8s charm does not communicate with the unix socket
             port=port,
             roles={RoleNames.ADMIN},
-            tls_external=self.tls.external_enabled,
-            tls_internal=self.tls.internal_enabled,
+            tls_enabled=self.tls.external_enabled,
+            tls_external_keyfile=self.paths.ext_pem_file,
+            tls_external_ca=self.paths.ext_ca_file,
         )
 
     @property
@@ -865,3 +764,23 @@ class CharmState(Object, StatusesStateProtocol):
         return self.mongos_config
 
     # END: Configuration accessors
+
+    def get_secret_from_id(self, secret_id: str) -> dict[str, str]:
+        """Resolve the given id of a Juju secret and return the content as a dict.
+
+        Args:
+            secret_id (str): The id of the secret.
+
+        Returns:
+            dict: The content of the secret.
+        """
+        if not secret_id.startswith("secret:"):
+            raise ValueError(f"Invalid secret URI '{secret_id}'. It must start with 'secret:'")
+        try:
+            secret_content = self.charm.model.get_secret(id=secret_id).get_content(refresh=True)
+        except SecretNotFoundError:
+            raise SecretNotFoundError(f"The secret '{secret_id}' does not exist.")
+        except ModelError:
+            raise
+
+        return secret_content
