@@ -31,7 +31,7 @@ from single_kernel_mongo.exceptions import (
 from single_kernel_mongo.utils.helpers import hostname_from_hostport, hostname_from_shardname
 from single_kernel_mongo.utils.mongo_config import MongoConfiguration
 from single_kernel_mongo.utils.mongo_error_codes import MongoErrorCodes
-from single_kernel_mongo.utils.mongodb_users import DBPrivilege, SystemDBS
+from single_kernel_mongo.utils.mongodb_users import DBPrivilege
 
 logger = logging.getLogger(__name__)
 
@@ -319,18 +319,6 @@ class MongoConnection:
         logger.debug("rs_config: %r", rs_config["config"])
         self.client.admin.command("replSetReconfig", rs_config["config"])
 
-    def get_databases(self) -> set[str]:
-        """Return list of all non-default databases."""
-        databases: list[str] = self.client.list_database_names()
-        return {db for db in databases if db not in SystemDBS}
-
-    def drop_database(self, database: str):
-        """Drop a non-default database."""
-        if database in SystemDBS:
-            logger.info(f"Not dropping system DB {database}.")
-            return
-        self.client.drop_database(database)
-
     def get_users(self) -> set[str]:
         """Add a new member to replica set config inside MongoDB."""
         users_info = self.client.admin.command("usersInfo")
@@ -403,7 +391,7 @@ class MongoConnection:
         Raises:
             ConfigurationError, OperationFailure
         """
-        shard_hosts = [f"{host}:{shard_port}" for host in shard_hosts]
+        shard_hosts = [f"{host}:{shard_port.value}" for host in shard_hosts]
         if shard_name in self.get_shard_members():
             logger.info("Skipping adding shard %s, shard is already in cluster", shard_name)
             return
@@ -678,7 +666,7 @@ class MongoConnection:
             if shard_name == shard_to_ignore:
                 continue
 
-            current_free_space = shard_storage_info["freeStorageSize"]
+            current_free_space = shard_storage_info["totalFreeStorageSize"]
             if current_free_space > candidate_free_space:
                 candidate_shard = shard_name
                 candidate_free_space = current_free_space
